-- ********************************************************************************
-- *  Copyright (C) Thorsten Gaertner, Oststeinbek / Germany 2004                 *
-- ********************************************************************************
--
-- ********************************************************************************
-- *  This source file is provided "as is" and WITHOUT any express or implied     *
-- *  warranties, that this source file is                                        *
-- *  1. free from any claims of infringement,                                    *
-- *  2. the merchantability or fitness for a particular purpose.                 *
-- ********************************************************************************
--
-- ********************************************************************************
-- *  File        : E2_CLK_GEN_FROM_E3.vhd                                        *
-- *  Version     : 1.0                                                           *
-- *  Target      : Plain VHDL                                                    *
-- *  Description : Generation of E2 clock 8448 kHz from E3 clock 34368 kHz       *
-- ********************************************************************************

library IEEE;
use     IEEE.STD_LOGIC_1164.ALL;
use     IEEE.STD_LOGIC_ARITH.ALL;
use     IEEE.STD_LOGIC_UNSIGNED.ALL;

-- ********************************************************************************

entity E2_CLK_GEN_FROM_E3 is
  port (
    E3_REF_CLK     : in  std_logic;  -- Framer Clock E3
    RESET          : in  std_logic;  -- Reset
    E2_REF_CLK     : out std_logic;  -- Framer Clock E2
    E2_REF_CLK_EN  : out std_logic   -- Framer Clock E2 Enable
  );
end E2_CLK_GEN_FROM_E3;

-- ********************************************************************************

architecture E2_CLK_GEN_FROM_E3 of E2_CLK_GEN_FROM_E3 is

-- ********************************************************************************

signal CLK     : std_logic;
signal counter : std_logic_vector (7 downto 0);

-- ********************************************************************************

begin

-- ********************************************************************************

CLK        <= E3_REF_CLK;
E2_REF_CLK <= E3_REF_CLK;

-- ********************************************************************************
-- * Generation of the E2_REF_CLK_EN signal :                                     *
-- * It is at 1 level while 8448 of 34368 cycles                                  *
-- ********************************************************************************
-- * 34368 : 8448 = 4,06818181 = 4 1/14,666                                       *
-- * 17184 : 4224                                                                 *
-- *  8592 : 2112                                                                 *
-- *  4296 : 1056                                                                 *
-- *  2148 :  528                                                                 *
-- *  1074 :  264                                                                 *
-- *   537 :  132                                                                 *
-- *   358 :   88                                                                 *
-- *   179 :   44                                                                 *
-- ********************************************************************************
-- * Base counter counts from 0 to 178                                            *
-- * The E2_REF_CLK_EN signal is deviated from the counter reading                *
-- * 1 when counter at 000,004,008,012,016,020,024,028,032,036,040,044,048,052,056*
-- * 1 when counter at 061,065,069,073,077,081,085,089,093,097,101,105,109,113,117*
-- * 1 when counter at 122,126,130,134,138,142,146,150,154,158,162,166,170,174    *
-- * 0 when counter at any other count                                            *
-- ********************************************************************************

E2_CLK_GEN : process (CLK, RESET)
begin
  if (RESET = '1') then

    counter       <= (others => '0');
    E2_REF_CLK_EN <= '0';

  elsif rising_edge (CLK) then

    if (counter = 178)
      then counter <= "00000000";
      else counter <= counter + 1;
    end if;

    if ((counter >=   0) and (counter <=  56) and (counter (1 downto 0) = "00")) or
       ((counter >=  61) and (counter <= 117) and (counter (1 downto 0) = "01")) or
       ((counter >= 122) and (counter <= 174) and (counter (1 downto 0) = "10"))
         then  E2_REF_CLK_EN <= '1';
         else  E2_REF_CLK_EN <= '0';
    end if;

  end if;
end process;

-- ********************************************************************************

end E2_CLK_GEN_FROM_E3;

-- ********************************************************************************
-- List Of Changes
-- ===============
-- DATE        VERSION  NAME  CHANGE
-- 10.07.2004  1.0      ThG   Creation
-- ********************************************************************************
